@@ -4,17 +4,15 @@ Structural variant collapser
 Will collapse all variants within sizemin/max that match over thresholds
 All variants outside size boundaries will be placed into the output
 """
-# pylint: disable=too-many-statements, no-member
+# pylint: disable=too-many-statements
 import os
 import sys
 import json
 import logging
 import argparse
-from functools import cmp_to_key
 from collections import defaultdict, namedtuple
 
 import pysam
-import pyfaidx
 
 import truvari
 from truvari.bench import edit_header as bench_edit_header
@@ -23,7 +21,7 @@ MATCHRESULT = namedtuple("matchresult", ("score seq_similarity size_similarity "
                                          "ovl_pct size_diff start_distance "
                                          "end_distance match_entry"))
 COLLAPENTRY = namedtuple("collapentry", ("entry match match_id key"))
-   
+
 def parse_args(args):
     """
     Pull the command line parameters
@@ -44,7 +42,7 @@ def parse_args(args):
                         help="Where collapsed variants are written (collapsed.vcf)")
     parser.add_argument("-f", "--reference", type=str, default=None,
                         help="Indexed fasta used to call variants")
-    parser.add_argument("-k", "--keep", choices=["first", "maxqual"], default="first",
+    parser.add_argument("-k", "--keep", choices=["first", "maxqual", "common"], default="first",
                         help="When collapsing calls, which one to keep (%(default)s)")
     parser.add_argument("--debug", action="store_true", default=False,
                         help="Verbose logging")
@@ -62,13 +60,16 @@ def parse_args(args):
                         help="Minimum pct reciprocal overlap (%(default)s) for DEL events")
     thresg.add_argument("-t", "--typeignore", action="store_true", default=False,
                         help="Variant types don't need to match to compare (%(default)s)")
+    thresg.add_argument("--use-lev", action="store_true",
+                        help="Use the Levenshtein distance ratio instead of edlib editDistance ratio (%(default)s)")
+
     parser.add_argument("--hap", action="store_true", default=False,
                         help="Collapsing a single individual's haplotype resolved calls (%(default)s)")
     parser.add_argument("--chain", action="store_true", default=False,
                         help="Chain comparisons to extend possible collapsing (%(default)s)")
     parser.add_argument("--null-consolidate", type=str, default=None,
                         help=("Comma separated list of FORMAT fields to consolidate into the kept "
-                            "entry by taking the first non-null from all neighbors (%(default)s)"))
+                              "entry by taking the first non-null from all neighbors (%(default)s)"))
     filteg = parser.add_argument_group("Filtering Arguments")
     filteg.add_argument("-s", "--sizemin", type=int, default=50,
                         help="Minimum variant size to consider for comparison (%(default)s)")
@@ -80,7 +81,7 @@ def parse_args(args):
     args = parser.parse_args(args)
     if args.pctsim != 0 and not args.reference:
         parser.error("--reference is required when --pctsim is set")
-    
+
     if args.null_consolidate is not None:
         args.null_consolidate = args.null_consolidate.split(',')
 
@@ -108,30 +109,12 @@ def collapse_edit_header(my_vcf):
     header = output_edit_header(my_vcf, header)
     return header
 
-def annotate_tp(entry, match_result):
-    """
-    Add the matching annotations to a vcf entry
-    match_score, match_pctsim, match_pctsize, match_ovlpct, match_szdiff, \
-                    match_stdist, match_endist, match_entry
-    """
-    entry.info["PctSeqSimilarity"] = match_result.seq_similarity
-    entry.info["PctSizeSimilarity"] = match_result.size_similarity
-    entry.info["PctRecOverlap"] = match_result.ovl_pct
-    entry.info["SizeDiff"] = match_result.size_diff
-    entry.info["StartDistance"] = match_result.start_distance
-    entry.info["EndDistance"] = match_result.end_distance
-
 def check_params(args):
     """
     Checks parameters as much as possible.
     All errors are written to stderr without logging since failures mean no output
     """
     check_fail = False
-    if args.output and os.path.exists(args.output):
-        logging.error("Output '%s' already exists", args.output)
-        check_fail = True
-    elif not args.output:
-        args.output = "/dev/stdout"
     if not os.path.exists(args.input):
         check_fail = True
         logging.error("File %s does not exist", args.input)
@@ -167,12 +150,12 @@ def setup_outputs(args):
     num_samps = len(outputs["o_header"].samples)
     if args.hap and num_samps != 1:
         logging.error("--hap mode requires exactly one sample. Found %d", num_samps)
-        exit(100)
+        sys.exit(100)
     outputs["output_vcf"] = pysam.VariantFile(args.output, 'w', header=outputs["o_header"])
     outputs["collap_vcf"] = pysam.VariantFile(args.collapsed_output, 'w', header=outputs["c_header"])
     return outputs
 
-def match_calls(base_entry, comp_entry, astart, aend, sizeA, sizeB, reference, args, outputs):
+def match_calls(base_entry, comp_entry, astart, aend, sizeA, sizeB, reference, args):
     """
     Compare the base and comp entries.
     We provied astart...sizeA because we've presumably calculated it before
@@ -200,7 +183,7 @@ def match_calls(base_entry, comp_entry, astart, aend, sizeA, sizeB, reference, a
         return True
 
     if args.pctsim > 0:
-        seq_similarity = truvari.entry_pctsim_lev(base_entry, comp_entry, reference, buf_len=args.buffer)
+        seq_similarity = truvari.entry_pctsim(base_entry, comp_entry, reference, args.buffer, args.use_lev)
         if seq_similarity < args.pctsim:
             logging.debug("%s and %s sequence similarity is too low (%f)", str(
                 base_entry), str(comp_entry), seq_similarity)
@@ -235,14 +218,19 @@ def edit_collap_entry(entry, match, match_id, outputs):
     truvari.bench.annotate_tp(new_entry, match)
     return new_entry
 
-def same_hap(entryA, entryB):
+def hap_resolve(entryA, entryB):
     """
-    Returns if two entry's first sample is on the same haplotype
-    by using GT phasing information
+    Returns true if the calls' genotypes suggest it can be collapsed
+    i.e. if either call is HOM, they can't be collapsed.
+    If calls are on the same haplotype (1/0 & 1/0), they cannot be collapsed
     """
     gtA = entryA.samples[0]["GT"]
     gtB = entryB.samples[0]["GT"]
-    return gtA == gtB
+    if gtA == (1, 1) or gtB == (1, 1):
+        return False
+    if gtA == gtB:
+        return False
+    return True
 
 def select_best(neighs):
     """
@@ -254,17 +242,12 @@ def select_best(neighs):
         if n[1].score > max_score:
             max_score = n[1].score
             max_score_pos = pos
-    i = len(neighs) - 1
-    while i > max_score_pos:
-        neighs.pop(i)
-        i -= 1
-    for i in range(max_score_pos):
-        neighs.pop(0)
+    return [neighs[max_score_pos]]
 
 def edit_output_entry(entry, neighs, match_id, hap, outputs, nullconso=None):
     """
     Edit the representative entry that's going to placed in the output vcf
-    If hap, consolidate with haplotype mode and edit the neighs in-place 
+    If hap, consolidate with haplotype mode and edit the neighs in-place
     down to only the single best match.
     Force is a set of FORMAT fields that will be checked for nulls and will overwrite between entries
     """
@@ -273,12 +256,14 @@ def edit_output_entry(entry, neighs, match_id, hap, outputs, nullconso=None):
 
     new_entry = truvari.copy_entry(entry, outputs["o_header"])
     new_entry.info["CollapseId"] = match_id
-    new_entry.info["NumCollapsed"] = len(neighs)
 
     if hap:
-        select_best(neighs)
+        neighs = select_best(neighs)
         new_entry.samples[0]["GT"] = (1, 1)
-        return new_entry
+        new_entry.info["NumCollapsed"] = len(neighs)
+        return new_entry, neighs
+
+    new_entry.info["NumCollapsed"] = len(neighs)
 
 
     # Update with the first genotyped sample's information
@@ -296,7 +281,7 @@ def edit_output_entry(entry, neighs, match_id, hap, outputs, nullconso=None):
                         assigned = True
                         fmt[key] = s_fmt[key]
                     idx += 1
-        
+
         # Now pull the rest based on GT presence
         if m_gt in [truvari.GT.NON, truvari.GT.REF]:
             idx = 0
@@ -309,15 +294,15 @@ def edit_output_entry(entry, neighs, match_id, hap, outputs, nullconso=None):
                     for key in [_ for _ in fmt if _ not in nullconso]:
                         try:
                             fmt[key] = s_fmt[key]
-                        except Exception as e:
-                            pass
+                        except Exception as e: # pylint: disable=broad-except
+                            logging.debug("Problem setting FORMAT field %s (%s)", key, str(e))
                 idx += 1
-    return new_entry
+    return new_entry, neighs
 
 def find_neighbors(base_entry, match_id, reference, matched_calls, args, outputs):
     """
     Find all matching neighbors of a call
-    returns a list of (edited_neighbors, match_id, 
+    returns a list of (edited_neighbors, match_id,
     """
     base_entry_size = truvari.entry_size(base_entry)
     thresh_neighbors = []
@@ -326,19 +311,19 @@ def find_neighbors(base_entry, match_id, reference, matched_calls, args, outputs
     fetch_end = aend + args.refdist + 1
     for comp_entry in outputs['seek_vcf'].fetch(base_entry.chrom, fetch_start, fetch_end):
         comp_key = truvari.entry_to_key('b', comp_entry)
-        # Don't collapse with anything that's already been seen
+        # Don't collapse with anything that's already been matched
         # Which will include looking at itself
         if matched_calls[comp_key]:
             continue
-        
+
         sizeB = truvari.entry_size(comp_entry)
         if sizeB < args.sizemin or sizeB > args.sizemax or (args.passonly and truvari.filter_value(comp_entry)):
             continue
 
-        if args.hap and same_hap(base_entry, comp_entry):
+        if args.hap and not hap_resolve(base_entry, comp_entry):
             continue
 
-        mat = match_calls(base_entry, comp_entry, astart, aend, base_entry_size, sizeB, reference, args, outputs)
+        mat = match_calls(base_entry, comp_entry, astart, aend, base_entry_size, sizeB, reference, args)
         if isinstance(mat, bool):
             continue
 
@@ -346,7 +331,7 @@ def find_neighbors(base_entry, match_id, reference, matched_calls, args, outputs
         # mark this entry as being matched already
         if not args.hap:
             matched_calls[comp_key] = True
-        
+
         thresh_neighbors.append(COLLAPENTRY(comp_entry, mat, match_id, comp_key))
     return thresh_neighbors
 
@@ -356,11 +341,11 @@ def select_maxqual(entry, neighs):
     """
     max_qual = entry.qual
     max_qual_idx = -1
-    for pos, i in enumerate(neighs):
+    for pos, val in enumerate(neighs): # pylint: disable=unused-variable
         cmp_qual = neighs[pos].entry.qual
         if cmp_qual > max_qual:
             max_qual_idx = pos
-    
+
     if max_qual_idx == -1:
         base_entry = entry
     else:
@@ -370,6 +355,27 @@ def select_maxqual(entry, neighs):
         neighs[max_qual_idx] = COLLAPENTRY(entry, swap.match, swap.match_id, key)
     return base_entry, neighs
 
+def select_common(entry, neighs):
+    """
+    Swap the entry with the one containing the highest MAC
+    """
+    most_mac = truvari.allele_freq_annos(entry)["MAC"]
+    most_mac_idx = -1
+    for pos, val in enumerate(neighs): # pylint: disable=unused-variable
+        cmp_mac = truvari.allele_freq_annos(neighs[pos].entry)["MAC"]
+        if cmp_mac > most_mac:
+            most_mac_idx = pos
+
+    if most_mac_idx == -1:
+        base_entry = entry
+    else:
+        swap = neighs[most_mac_idx]
+        base_entry = swap.entry
+        key = truvari.entry_to_key('b', entry)
+        neighs[most_mac_idx] = COLLAPENTRY(entry, swap.match, swap.match_id, key)
+    return base_entry, neighs
+
+
 def collapse_main(cmdargs):
     """
     Main entry point for running Truvari collapse
@@ -395,7 +401,7 @@ def collapse_main(cmdargs):
         base_key = truvari.entry_to_key('b', base_entry)
         if matched_calls[base_key]:
             continue
-        
+
         sizeA = truvari.entry_size(base_entry)
         if sizeA < args.sizemin or sizeA > args.sizemax or (args.passonly and truvari.filter_value(base_entry)):
             outputs["output_vcf"].write(base_entry)
@@ -404,7 +410,7 @@ def collapse_main(cmdargs):
 
         # First time seeing it, last time using it
         matched_calls[base_key] = True
-        
+
         thresh_neighbors = []
         new_neighs = find_neighbors(base_entry, match_id, reference, matched_calls, args, outputs)
         thresh_neighbors.extend(new_neighs)
@@ -418,17 +424,19 @@ def collapse_main(cmdargs):
 
         if args.keep == "maxqual":
             base_entry, thresh_neighbors = select_maxqual(base_entry, thresh_neighbors)
-            
+        if args.keep == "common":
+            base_entry, thresh_neighbors = select_common(base_entry, thresh_neighbors)
+
         if thresh_neighbors:
-            new_entry = edit_output_entry(base_entry, thresh_neighbors, match_id, args.hap, outputs, args.null_consolidate)
+            new_entry, thresh_neighbors = edit_output_entry(base_entry, thresh_neighbors, match_id, args.hap, outputs, args.null_consolidate)
             outputs["output_vcf"].write(new_entry)
-            
+
             # Also want to record the original beside its' collapse neighbors
             new_entry = truvari.copy_entry(base_entry, outputs["c_header"])
             new_entry.info["CollapseId"] = match_id
             new_entry.info["NumCollapsed"] = len(thresh_neighbors)
             outputs["collap_vcf"].write(new_entry)
-            
+
             output_cnt += 1
             kept_cnt += 1
             for neigh in thresh_neighbors:
@@ -444,10 +452,7 @@ def collapse_main(cmdargs):
     logging.info("%d calls written to output", output_cnt)
     logging.info("%d collapsed calls", collap_cnt)
     logging.info("%d representative calls kept", kept_cnt)
-    
+
     # Close to flush vcfs
     close_outputs(outputs)
-    logging.info("Finished")
-
-if __name__ == '__main__':
-    collapse_main(sys.argv[1:])
+    logging.info("Finished collapse")
